/** File name: AdjacencyList.java
 *  This is the class for adjacency list.
 *
 */

import java.util.Vector ;

/** This class represents the edges and vertices that are adjacent to another vertex
 */
public class AdjacencyList {
    
    Vector vertices = new Vector() ;	// the adjacent vertices of v
    Vector edges = new Vector() ;	// the edges that lead to these veticies
    
    
    /** constructor for creating an AdjacencyList object 
     */
    public AdjacencyList() {
    }
    
    
    /** adds an adjacency vertex to adj.
     * @param v the adjacent vertex
     * @param e the edge that leads to v
     */
    public void addAdjacentVertex(Vertex v, Edge e) {
        edges.add(e);
        vertices.add(v) ;
    }
    
    /** returns the number of adjacent vertices
     * @return the number of adjacent vertices
     */    
    public int length() {
        return edges.size() ;
    }
    
    /** returns the vertex at index i
     * @return the vertex
     * @param i the index
     */    
    public Vertex getVertex(int i) {
        if(i >= edges.size() ) {
            return null ;
        } else {
            return (Vertex) vertices.get(i) ;
        }
    }
    
    /** gets the edge at this index
     * @return the edge
     * @param i the index
     */    
    public Edge getEdge(int i) {
        return (Edge) edges.get(i) ;
    }
    
    /** gets the edge for the specified vertex, implemented in constant time
     * @return the edge
     * @param v the vertex
     */    
    public Edge getEdgeToVertex(Vertex v) {
        for (int i = 0; i < edges.size() ; i++) {
            if(vertices.get(i) == v)
                return (Edge) edges.get(i) ;
        }
        return null ;
    }
    
    
    /** returns the string representation of this vertex
     * @return a string
     */    
    public String toString() {
        
        String str = "";
        for (int i=0; i < edges.size() ; i++) {
            str = str + ( ( Vertex) vertices.get(i)).getLabel() + ", ";
        }
        return  str;
    }
    
    
}

