

/** This class represents an edge in the graph. Edges have a capacity,
 * a flow and a residual (capacity - flow).
 *
 * Each edge also has a complement, this is the edge going the opposite dirrection
 * to the current edge. ie. if edge e goes from u to v. The complement of e goes from v to u.
 *
 * The flow through the complement should always be the negative of the flow through the edge
 */
public class Edge {
    
    private int flow = 0 ;
    private int capacity  ;
    
    private Edge complement ;
    
    // we have to deal with infinity here for multisource
    /** Creates a new edge
     * @param capacity The capacity of this edge
     */
    public Edge( int capacity ) {
        this.capacity = capacity ;
    }
    
    /** Returns the residual (remaining capacity left in this edge)
     * @return the residual
     */
    public int getRes() {
        return (capacity == -1 ? -1 : capacity - flow ) ;
    }
    
    /** Returns the flow going through this edge
     * @return the current flow through this edge
     */
    public int getFlow() {
        return flow ;
    }
    
    /** Sets the flow through this edge
     * @param flow the flow thru this edge
     */
    public void setFlow(int flow) {
        this.flow = flow ;
    }
    
    
    /** Returns the total capacity of this edge
     * @return the total capacity of this edge
     */
    public int getCap() {
        return capacity ;
    }
    
    
    /** Sets the capacity of this edge
     * @param cap the new capacity
     */
    public void setCap(int cap) {
        this.capacity = cap ;
    }
    
    /** Returns the complement edge
     * @return the complementaty edge
     */
    public Edge getComplement() {
        return complement ;
        
    }
    
    /** Sets the complementary edge
     * @param e the edge that is the complement to this one
     */
    public void setComplement(Edge e) {
        complement = e ;
    }
    
}
