/**
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/** An implmentation of the MaxFlowAlgorithm, using the Edmonds Karp
 */
public class Edmonds extends MaxFlowAlgorithm {

    
    /** Used for the command line program.
     * @param args the commaind line arguments
     */     
    public static void main(String args[]) throws Exception {

        if(args.length == 0) {
            System.out.println("Usage: java Edmonds FILE") ;
            return ;
        }
	
	Graph g = Parser.parse(args[0]) ;
	g.normalise() ;

        Edmonds m = new Edmonds(g) ;
        
	Result r  = m.run(true) ;
        
        m.printDetails(r) ;
    }


    /** create a new object
     * @param g the graph to operate on
     */    
    public Edmonds(Graph g) {
        super(g) ;
        done = false ;
    }
    
    /** Runs the algorithm on the graph. The algorithm can either be run step wise
     * , or until it completes. In this implemation, one step is the addition of one augmenting
     * path.
     * @param cont if true, the algorithm runs to completion. if false, it will run step -wise.
     * @return a result object
     * @throws Exception if an error occurs within the algorithm
     */    
    public Result run(boolean cont) throws Exception {
                
	BFS bfs = new BFS(g) ;
        Path p ;
                
	// START MAIN LOOP
	do {
            // do BFS
            bfs.search(g.getSource()) ;
	
            // check for an augmenting path
            p = Path.fromGraph(g) ;

            if (p == null ) {
		// no remaining augmenting paths to the sink
		// time to finish
                done = true ;
                break ;
            }
                
            // loop thru all edges in path
            for (int i =0 ; i < p.length() ; i++ ) {
                Edge e = p.getEdge(i) ;

		// set new flow
		e.setFlow( e.getFlow() + p.getBottleneck() ) ;

		// set flow for the oppositte edge
		Edge e2 = e.getComplement() ;
		e2.setFlow(e2.getFlow() - p.getBottleneck() ) ;
            }

	} while (cont) ;
                              
        Result r = new Result(calcFlow(), p == null ) ;
        r.setPath(p) ;
        return r ;	
    }

} // end class
