

/* TODO
 *  get rid of all the for loops
 *  use some osrt of lookup to give constant time
 */

import java.util.Vector ;

/** this class represent a graph (network). A network has vertices,
 * and each vertex has a corrosponding adjacentcy list.
 */
public class Graph {
    private Vertex[] vertices;
    AdjacencyList[] adjList;
    private Vertex source ;			// the one true source
    private Vector sources = new Vector() ;	// the defined sources
    private Vertex sink ;			// the one true sink
    private Vector sinks = new Vector() ; 	// defined sinks
    
    /** create a new graph
     * @param v an array of vertices
     * @param adjList an array of adjLists
     */    
    public Graph(Vertex[] v, AdjacencyList[] adjList) {
        vertices = v;
        this.adjList = adjList;
    }
    
    // this is used to add a vertex to a pre constructed graph
    // useful if we need to add a "super source" when normalising
    /** adds a vertex to the graph
     * @param v the vertex to add
     * @param al the adjList for this vertex
     */    
    public void addVertex(Vertex v, AdjacencyList al) {
        int len = vertices.length ;
        Vertex[] tmp = new Vertex[len + 1];
        AdjacencyList[] tmp2 = new AdjacencyList[len + 1] ;
        
        for (int i=0; i < len; i++) {
            tmp[i] = vertices[i];
            tmp2[i] = adjList[i] ;
        }
        
        tmp[len] = v;
        tmp2[len] = al ;
        
        vertices = tmp;
        adjList = tmp2 ;
    }
    
    /** returns the vertex whose label is the same as
     * the given label.  If not found then the return null.
     * @param label the label
     * @return the vertex
     */
    public Vertex getVertex(String label) {
        for (int i=0; i < vertices.length; i++)
            if (label.equals(vertices[i].getLabel()))
                return vertices[i];
        return null;
    }
    
    /** returns the index of the specified vertex
     * @return the index
     * @param v the vertex to return the index for
     */
    public int getVertex(Vertex v) {
        for (int i=0; i < vertices.length; i++)
            if (v.getLabel().equals(vertices[i].getLabel()))
                return i;
        return -1;
    }
    
    /** returns the vertex at index i
     * @param i the index
     * @return the vertex at i
     */
    public Vertex getVertex(int i) {
        return vertices[i];
    }
    
    
    /** returns the list containing the adjacent vertices of a given vertex
     * @return the adjList
     * @param v the vertex to return the adjList for
     */
    public AdjacencyList getAdjList(Vertex v) {
        for (int i=0; i < vertices.length; i++)
            if (v == vertices[i])
                return adjList[i];
        return null;
        
    }
    
    /** returns the list containing the adjacent vertices of a given vertex
     * @return the adjList for vertex i
     * @param i the index
     */
    public AdjacencyList getAdjList(int i) {
        return adjList[i];
        
    }
    
    /** returns the number of vertices in the graph
     * @return the number of vertices
     */    
    public int getNVertices() {
        return vertices.length;
    }
    
    
    /** prints the vertices in this graph
     */    
    public void printVertices() {
        for (int i=0; i < vertices.length; i++)
            System.out.println(vertices[i]);
    }
    
    /** returns a string representation of the graph
     * @return a string
     */    
    public String toString() {
        String str = "";
        for (int i=0; i < vertices.length; i++)
            str = str + adjList[i].toString() +  "\n";
        return str;
    }
    
   
    
    /** returns the source of the graph
     * @return the source
     */    
    public Vertex getSource() {
        return source ;
    }
    
    /** returns the sink of the graph
     * @return the sink
     */    
    public Vertex getSink() {
        return sink ;
    }
    
    /** 'Normalises' the graph. The following steps are performed:
     *
     * check for multiple sources, and correct if necessary
     * check for mulitple sinks, and correct if necessary
     * check that each edge has a complement, create extra edges if required, and set each edges complement
     */    
    public void normalise() throws Exception {
        
        // collect all source and sink verticies
        for( int i =0 ; i < getNVertices() ; i++) {
            Vertex v1 = getVertex(i) ;
            if (v1.isSink())
                sinks.add(v1) ;
            if (v1.isSource() ) {
                sources.add(v1) ;
            }
        }
        
        if( sources.size() == 0 || sinks.size() == 0 ) {
            System.out.println("source or sink not defined") ;
            throw new Exception("Source of sink missing!!") ;
        }
        
        // check for multi source
        if(sources.size() == 1) {
            source = (Vertex) sources.get(0) ;
        } else {
            // have to create super source
            Vertex src = new Vertex(String.valueOf(getNVertices()) ) ;
            AdjacencyList al = new AdjacencyList() ;
            
            for (int i=0; i < sources.size() ; i++ ) {
                Vertex v = (Vertex) sources.get(i) ;
                AdjacencyList al1 = getAdjList(v ) ;
                
                int maxCap = 0 ;
                for(int j =0 ; j < al1.length() ; j++) {
                    maxCap += al1.getEdge(j).getCap() ;
                }
                
                Edge e = new Edge(maxCap) ;
                al.addAdjacentVertex((Vertex) sources.get(i) , e) ;
            }
            addVertex(src,al) ;
            source = src ;
            
        }
        
        // check for multi sink
        if(sinks.size() == 1) {
            sink = (Vertex)  sinks.get(0) ;
        } else {
            // have to create super sink
            Vertex snk = new Vertex(String.valueOf(getNVertices()) ) ;
            
            for(int i=0; i < sinks.size() ; i++ ) {
                AdjacencyList al = getAdjList((Vertex) sinks.get(i) ) ;
                al.addAdjacentVertex(snk, new Edge(-1) ) ;
                
            }
            
            addVertex(snk,new AdjacencyList()) ;
            sink = snk ;
            
        }
        
        // now set complement links
        for (int i=0; i < vertices.length; i++) {
            Vertex src = vertices[i] ;
            AdjacencyList l = adjList[i] ;
            
            for (int j = 0; j < l.length() ; j++ ) {
                Edge e1 = l.getEdge(j) ;
                
                if(e1 != null) {
                    Vertex dst = l.getVertex(j) ;
                    
                    Edge e2 = getAdjList(dst).getEdgeToVertex(src) ;
                    
                    if(e2 == null) {
                        e2 = new Edge(0) ;
                        getAdjList(dst).addAdjacentVertex(src, e2) ;
                    }
                    
                    e1.setComplement(e2) ;
                    e2.setComplement(e1) ;
                    
                }
            }
        }
    }
    
    
}
