
import java.awt.Graphics ;
import java.awt.Dimension ;

/** this is the graphical representation of an Edge. it extends javax.swing.jLabel
 */
public class gEdge extends javax.swing.JComponent  {
    
    private gVertex src;
    private gVertex dst ;
    private Edge e ;
    private int x,y,w,h ;
    private gVertex top, left ;
    private boolean inAugPath ;
    
    private static final int OFFSET = 20 ;
    
    /** Creates a new instance of gEdge */
    public gEdge(gVertex src, gVertex dst, Edge e) {
        super() ;
        this.src = src;
        this.dst = dst ;
        this.e = e ;
        
        // register for events
        src.addEdge(this) ;
        dst.addEdge(this) ;
        
        calcSize() ;
        setLocation(x,y) ;
        
    }
    
    /** marks this edge as being on the augmentation path
     */
    public void setAugPath(boolean b) {
        inAugPath = b ;
    }
    
    /** returns true if in augmentation path , false otherwise
     */
    public boolean isInAugPath() {
        return inAugPath ;
    }
    
    /** returns the x position on this edge
     */
    public int getX() {
        return x ;
    }
    
    /** returns the y position on this edge
     */
    public int getY() {
        return y ;
    }
    
    // called everytime a vertex moves
    public void update() {
        calcSize() ;
        this.reshape(x,y,w,h) ;
        repaint() ;
    }
    
    
    protected void paintComponent(Graphics g ) {
        
        /*
         * Truth table for calculating co-ords from position of src and dst
         *
         * left== src   top==src    x1  y1  x2  y2
         *      y           y       0   0   w   h
         *      y           n       w   0   0   h
         *      n           n       w   h   0   0
         *      n           y       0   h   w   0
         */
        int x1 = (left == src ? OFFSET : w-OFFSET ) ;
        int x2 = (left == src ? w-OFFSET : OFFSET ) ;
        int y1 = (top == src ? OFFSET : h-OFFSET ) ;
        int y2 = (top == src ? h-OFFSET : OFFSET ) ;
        
        
        if(e.getCap() != 0 ) {
            if(inAugPath) {
                g.setColor( new java.awt.Color(0,255,0) ) ;
            }
            g.drawLine(x1,y1, x2,y2 ) ;
            
            g.setColor( new java.awt.Color(0,0,0) ) ;
            if(e.getCap() == -1) 
                g.drawString(e.getFlow() + "/inf"  , w/2 , h/2 ) ;   
            else
                g.drawString(e.getFlow() + "/" + e.getCap()  , w/2 , h/2 ) ;
        }
    }
    
    private void calcSize() {
        left = src.getX() < dst.getX() ? src : dst ;
        top = src.getY() < dst.getY() ? src : dst ;
        
        // find top left
        x = Math.min(src.getX(), dst.getX() ) ;
        y = Math.min(src.getY(), dst.getY() ) ;
        
        // this assume the boxes are the same width, should prob fix
        x += left.getWidth() / 2;
        y += top.getHeight() / 2 ;
        
        x-=OFFSET ;
        y-=OFFSET ;
        
        // find size
        w = Math.abs(src.getX() - dst.getX() ) ;
        h = Math.abs(src.getY() - dst.getY() ) ;
        
        w += 2*OFFSET;
        h += 2*OFFSET ;
        
    }
    
    public Dimension getMinimumSize() {
        Dimension d = super.getMinimumSize() ;
        System.out.println(d) ;
        calcSize() ;
        return new Dimension(w,h) ;
    }
    
    public Dimension getPreferredSize() {
        calcSize() ;
        return new Dimension(w,h) ;
    }
    
    public Dimension getSize(Dimension d) {
        d.setSize(w,h) ;
        return d ;
    }
       
}
